@@ -2,25 +2,14 @@
 #
 # SPDX-License-Identifier: EUPL-1.2
 
-import statsd  # type: ignore
-from .. import Base
+import minty
 from ..infrastructure import InfrastructureFactory
 from ..repository import RepositoryBase, RepositoryFactory
 from .events import Event, EventService, event
 from abc import ABC, abstractmethod
 from copy import copy
 from functools import partial
-from typing import (
-    Any,
-    Dict,
-    Iterable,
-    List,
-    Protocol,
-    Set,
-    Type,
-    TypedDict,
-    cast,
-)
+from typing import Any, Dict, Iterable, List, Protocol, Set, Type, TypedDict
 from uuid import UUID
 
 __all__ = [
@@ -54,7 +43,7 @@ class UserInfo:
         self.permissions = permissions
 
 
-class SplitCommandBase(Base, ABC):
+class SplitCommandBase(minty.Base, ABC):
     """
     Base class for 'command' parts of domains.
 
@@ -123,7 +112,7 @@ class SplitCommandBase(Base, ABC):
         return self.cmd.user_info.permissions.get(permission, False)
 
 
-class SplitQueryBase(Base, ABC):
+class SplitQueryBase(minty.Base, ABC):
     """
     Base class for 'query' parts of domains.
 
@@ -271,7 +260,7 @@ class DomainQueryContainer:
             raise AttributeError(name) from e
 
 
-class CommandBase(Base, ABC):
+class CommandBase(minty.Base, ABC):
     "Base class for 'command' parts of domains"
 
     def __init__(
@@ -302,7 +291,7 @@ class CommandBase(Base, ABC):
         )
 
 
-class QueryBase(Base, ABC):
+class QueryBase(minty.Base, ABC):
     "Base class for 'query' parts of domains"
 
     def __init__(self, repository_factory, context, user_uuid: UUID):
@@ -327,7 +316,7 @@ class QueryBase(Base, ABC):
         )
 
 
-class QueryMiddleware(Base, ABC):
+class QueryMiddleware(minty.Base, ABC):
     __slots__ = [
         "infrastructure_factory",
         "correlation_id",
@@ -382,7 +371,7 @@ class QueryMiddleware(Base, ABC):
         pass
 
 
-class MiddlewareBase(Base, ABC):
+class MiddlewareBase(minty.Base, ABC):
     __slots__ = [
         "infrastructure_factory",
         "event_service",
@@ -460,6 +449,29 @@ class CommandInfrastructureCleanup(MiddlewareBase):
             self.infrastructure_factory.flush_local_storage()
 
 
+class StatsdContextMiddleware(MiddlewareBase):
+    """Set (and unset) the current 'context' in the per-thread statsd object."""
+
+    def __call__(self, func):
+        try:
+            minty.STATSD.context = self.context.replace(".", "_")
+            func()
+        finally:
+            minty.STATSD.context = None
+
+
+class QueryStatsdContextMiddleware(QueryMiddleware):
+    """Set (and unset) the current 'context' in the per-thread statsd object"""
+
+    def __call__(self, func):
+        try:
+            minty.STATSD.context = self.context.replace(".", "_")
+            rv = func()
+            return rv
+        finally:
+            minty.STATSD.context = None
+
+
 class QueryInfrastructureCleanup(QueryMiddleware):
     """Set current event in infrastructure factory and clear it after the query is finished."""
 
@@ -471,7 +483,7 @@ class QueryInfrastructureCleanup(QueryMiddleware):
             self.infrastructure_factory.flush_local_storage()
 
 
-class QueryWrapper(Base):
+class QueryWrapper(minty.Base):
     """Wrapper class for query instances that applies middleware."""
 
     def __init__(self, query_instance, middleware: list, domain: str):
@@ -496,27 +508,18 @@ class QueryWrapper(Base):
 
             def wrapped(*args, **kwargs):
                 """Return attribute wrapped in middlewares."""
+                wrapped_func = partial(original_attribute, *args, **kwargs)
 
-                timer = cast(
-                    statsd.Timer,
-                    self.statsd.get_timer(
-                        f"query,domain={self.domain},query={attr}"
-                    ),
-                )
-
-                with timer.time():
-                    wrapped_func = partial(original_attribute, *args, **kwargs)
-
-                    for middleware in self.middleware:
-                        wrapped_func = partial(middleware, wrapped_func)
-                    return wrapped_func()
+                for middleware in self.middleware:
+                    wrapped_func = partial(middleware, wrapped_func)
+                return wrapped_func()
 
             return wrapped
         else:
             return original_attribute
 
 
-class CommandWrapper(Base):
+class CommandWrapper(minty.Base):
     """Wrapper class for command instances. Handles creation of Events."""
 
     def __init__(self, command_instance, domain: str):
@@ -552,20 +555,11 @@ class CommandWrapper(Base):
 
             def wrapped(*args, **kwargs):
                 """Return attribute wrapped in middlewares."""
+                wrapped_func = partial(original_attribute, **kwargs)
+                for middleware in self.middlewares:
+                    wrapped_func = partial(middleware, wrapped_func)
 
-                timer = cast(
-                    statsd.Timer,
-                    self.statsd.get_timer(
-                        f"command,domain={self.domain},query={attr}"
-                    ),
-                )
-
-                with timer.time():
-                    wrapped_func = partial(original_attribute, **kwargs)
-                    for middleware in self.middlewares:
-                        wrapped_func = partial(middleware, wrapped_func)
-
-                    wrapped_func()
+                wrapped_func()
                 return
 
             return wrapped
@@ -607,7 +601,7 @@ class CQRSDomainDescriptor(TypedDict):
     repository_factory_ro: RepositoryFactory
 
 
-class CQRS(Base):
+class CQRS(minty.Base):
     """Keep commands and queries separated.
 
     CQRS: Command Query Responsibility Separation
@@ -711,37 +705,37 @@ class CQRS(Base):
         )
 
         short_domain = domain.split(".")[-1]
-        timer = cast(
-            statsd.Timer,
-            self.statsd.get_timer(f"get_query_instance,domain={short_domain}"),
+
+        query_instance = self.domains[domain]["module"].get_query_instance(
+            self.domains[domain]["repository_factory_ro"],
+            context=context,
+            user_uuid=user_uuid,
         )
-        with timer.time():
-            query_instance = self.domains[domain]["module"].get_query_instance(
-                self.domains[domain]["repository_factory_ro"],
-                context=context,
-                user_uuid=user_uuid,
-            )
 
-            initialized_middleware = []
+        initialized_middleware = []
 
-            # Outermost layer: cleanup infrastructure when we're done
-            for mw in [*self.query_middleware, QueryInfrastructureCleanup]:
-                initialized_middleware.append(
-                    mw(
-                        infrastructure_factory=self.infrastructure_factory_ro,
-                        correlation_id=correlation_id,
-                        domain=domain,
-                        context=context,
-                        user_uuid=user_uuid,
-                    )
+        # Outermost layer: cleanup infrastructure when we're done
+        for mw in [
+            *self.query_middleware,
+            QueryInfrastructureCleanup,
+            QueryStatsdContextMiddleware,
+        ]:
+            initialized_middleware.append(
+                mw(
+                    infrastructure_factory=self.infrastructure_factory_ro,
+                    correlation_id=correlation_id,
+                    domain=domain,
+                    context=context,
+                    user_uuid=user_uuid,
                 )
-
-            wrapped_query = QueryWrapper(
-                query_instance=query_instance,
-                middleware=initialized_middleware,
-                domain=short_domain,
             )
 
+        wrapped_query = QueryWrapper(
+            query_instance=query_instance,
+            middleware=initialized_middleware,
+            domain=short_domain,
+        )
+
         return wrapped_query
 
     def get_command_instance(
@@ -768,45 +762,40 @@ class CQRS(Base):
         )
 
         short_domain = domain.split(".")[-1]
-        timer = cast(
-            statsd.Timer,
-            self.statsd.get_timer(
-                f"get_command_instance,domain={short_domain}"
-            ),
+
+        event_service = EventService(
+            correlation_id=correlation_id,
+            domain=domain,
+            context=context,
+            user_uuid=user_uuid,
         )
-        with timer.time():
-            event_service = EventService(
+
+        cmd_instance = self.domains[domain]["module"].get_command_instance(
+            self.domains[domain]["repository_factory"],
+            context=context,
+            user_uuid=user_uuid,
+            event_service=event_service,
+        )
+
+        cmd_wrapped = CommandWrapper(
+            command_instance=cmd_instance, domain=short_domain
+        )
+
+        # Always add some "outer layers" to clean up
+        for middleware in [
+            *self.command_wrapper_middleware,
+            CommandInfrastructureCleanup,
+            EventServiceCleanup,
+            StatsdContextMiddleware,
+        ]:
+            initialized_middleware = middleware(
+                infrastructure_factory=self.infrastructure_factory,
                 correlation_id=correlation_id,
                 domain=domain,
                 context=context,
                 user_uuid=user_uuid,
-            )
-
-            cmd_instance = self.domains[domain]["module"].get_command_instance(
-                self.domains[domain]["repository_factory"],
-                context=context,
-                user_uuid=user_uuid,
                 event_service=event_service,
             )
-
-            cmd_wrapped = CommandWrapper(
-                command_instance=cmd_instance, domain=short_domain
-            )
-
-            # Always "outer layer" to clean up infrastructure
-            for middleware in [
-                *self.command_wrapper_middleware,
-                CommandInfrastructureCleanup,
-                EventServiceCleanup,
-            ]:
-                initialized_middleware = middleware(
-                    infrastructure_factory=self.infrastructure_factory,
-                    correlation_id=correlation_id,
-                    domain=domain,
-                    context=context,
-                    user_uuid=user_uuid,
-                    event_service=event_service,
-                )
-                cmd_wrapped.register_middleware(initialized_middleware)
+            cmd_wrapped.register_middleware(initialized_middleware)
 
         return cmd_wrapped
